"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var es2015_1 = __importDefault(require("estel-estree-builder/generated/es2015"));
var ramda_1 = __importDefault(require("ramda"));
var utils_1 = require("./utils");
exports.arrow = function () {
    var body = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        body[_i] = arguments[_i];
    }
    return es2015_1.default.arrowFunctionExpression(es2015_1.default.functionBody(utils_1.noNulls(body)), []);
};
exports.callFn = function (callee, args) {
    return es2015_1.default.callExpression(utils_1.isString(callee) ? es2015_1.default.identifier(callee) : callee, args || []);
};
exports.buildBeforeEach = function (propDefs) {
    var mocks = propDefs.map(function (p) {
        return es2015_1.default.assignmentExpression('=', es2015_1.default.identifier(p.mockName), p.mockVal);
    });
    return exports.callFn('beforeEach', [
        exports.arrow.apply(void 0, mocks),
    ]);
};
var space = function (c) { return ramda_1.default.repeat(' ', c).join(''); };
var s6 = space(6);
var s8 = space(8);
var propPad = "\n" + s8;
exports.buildRenderFunc = function (name, propTypes) {
    if (propTypes === void 0) { propTypes = []; }
    var propAssigns = propTypes.map(function (p) {
        return p.propName + "={" + p.mockName + "}";
    }).join(propPad);
    var propAssignsWithPad = propAssigns ? "" + propPad + propAssigns + propPad : ' ';
    return es2015_1.default.functionDeclaration(es2015_1.default.identifier('renderComponent'), [es2015_1.default.identifier('props')], es2015_1.default.functionBody([
        es2015_1.default.assignmentExpression('=', es2015_1.default.identifier('props'), es2015_1.default.logicalExpression('||', es2015_1.default.identifier('props'), es2015_1.default.objectExpression([]))),
        // return render(<Foo a=a, b=b, {...props})
        es2015_1.default.returnStatement(exports.callFn('shallow', [es2015_1.default.identifier("\n" + s6 + "<" + name + propAssignsWithPad + "{...props} />")])),
    ]));
};
exports.buildItBlock = function (desc, expressions) {
    return exports.callFn('it', [es2015_1.default.literal(desc), es2015_1.default.arrowFunctionExpression(es2015_1.default.functionBody(expressions), [])]);
};
exports.buildClass = function (className, superClass, methods) {
    var methodDefs = Object.keys(methods).map(function (k) {
        var def = methods[k];
        return es2015_1.default.methodDefinition(es2015_1.default.identifier(k), def, 'get');
    });
    return es2015_1.default.class(es2015_1.default.classBody(methodDefs), "ClassDeclaration", es2015_1.default.identifier(className), es2015_1.default.identifier(superClass));
};
exports.buildImportStmts = function (namedImportMaps, defaultImportMaps, exportSuiteTrees) {
    var imports = {};
    if (exportSuiteTrees) {
        exportSuiteTrees.forEach(function (t) {
            var sutImports = imports[t.inputModulePath] || {};
            var sutImportsNamed = sutImports.named || [];
            if (t.defaultSut) {
                sutImports.default = t.defaultSut;
            }
            if (t.namedSut) {
                sutImportsNamed.push(t.namedSut);
                sutImports.named = sutImportsNamed;
            }
            if (sutImports.named || sutImports.default) {
                imports[t.inputModulePath] = sutImports;
            }
        });
    }
    namedImportMaps.forEach(function (curr) {
        Object.keys(curr).forEach(function (k) {
            var namedList = curr[k];
            var existing = imports[k] || { named: [] };
            var prevList = existing.named || [];
            existing.named = utils_1.mergeArrays(namedList, prevList);
            imports[k] = existing;
        });
    });
    defaultImportMaps.forEach(function (curr) {
        Object.keys(curr).forEach(function (k) {
            var defaultName = curr[k];
            var existing = imports[k] || {};
            existing.default = defaultName;
            imports[k] = existing;
        });
    });
    var importStmts = Object.keys(imports).map(function (p) {
        var i = imports[p];
        var named = i.named;
        var namedBrackets = named ? "{ " + named.join(', ') + " } " : '';
        var def = utils_1.pad(i.default, named ? ',' : '');
        return es2015_1.default.identifier("import" + def + namedBrackets + "from '" + p + "'");
    });
    return importStmts;
};
exports.dot = function (target, prop) {
    return es2015_1.default.memberExpression(target, es2015_1.default.identifier(prop));
};
exports.assign = function (left, right) {
    return es2015_1.default.assignmentExpression('=', left, right);
};
exports.fnArgs = function (args) {
    var rest = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        rest[_i - 1] = arguments[_i];
    }
    return es2015_1.default.functionExpression(args, es2015_1.default.functionBody(utils_1.noNulls(rest)));
};
exports.fn = function () {
    var body = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        body[_i] = arguments[_i];
    }
    return exports.fnArgs([], body);
};
exports.vr = function (name, val) {
    return es2015_1.default.variableDeclaration([es2015_1.default.variableDeclarator(es2015_1.default.identifier(name), val)], "var");
};
exports.cnst = function (name, val) {
    return es2015_1.default.variableDeclaration([es2015_1.default.variableDeclarator(es2015_1.default.identifier(name), val)], "const");
};
exports.lt = function (name, val) {
    if (val === void 0) { val = undefined; }
    return es2015_1.default.variableDeclaration([es2015_1.default.variableDeclarator(es2015_1.default.identifier(name), val)], "let");
};
exports.raw = es2015_1.default.identifier;
//# sourceMappingURL=tree-builders.js.map